const path = require('path');
// const babylon = require('babylon');
const babelParse = require('@babel/parser');
const compiler = require('vue-template-compiler');
const html = require('html5parser');

exports.parser = function (content, filePath) {
  try {
    return attempt(
      () => _parse(content, filePath, 'script'),
      () => _parse(content, filePath, 'module')
    );
  } catch (err) {
    const ctx = getErrorContext(err, content);
    throw new Error(`Couldn't parse ${filePath}: ${err.message}${ctx}`);
  }
}

exports.parserTemplate = function () {

}

function attempt(...fns) {
  for (let i = 0; i < fns.length; i++) {
    try {
      return fns[i]();
    } catch (err) {
      if (i === fns.length - 1) throw err;
    }
  }
}

function _parse (content, filePath, sourceType) {
  let start = 1;
      realContent = content;
      lang = 'js';
  plugins = [
    'estree',
    // ['flow', { all: true }],
    'flowComments',
    'jsx',
    ['typescript', { allowJs: true }],
    'asyncGenerators',
    'classProperties',
    'classPrivateProperties',
    'classPrivateMethods',
    ['decorators', { decoratorsBeforeExport: true }],
    'doExpressions',
    'dynamicImport',
    'exportDefaultFrom',
    'exportNamespaceFrom',
    'logicalAssignment',
    'nullishCoalescingOperator',
    'objectRestSpread',
    'optionalCatchBinding',
    'optionalChaining',
    'partialApplication',
    ['pipelineOperator', { proposal: 'smart' }],
    'numericSeparator',
    'topLevelAwait'
  ];
  const extname = path.extname(filePath);
  if (extname === '.vue') {
    const compiled = _parseVueTemplate(content);
    realContent = compiled.content || '';
    start = compiled.start;
    lang = compiled.lang || lang;
  }
  if (extname === '.ts') {
    lang = 'ts';
  }
  if (!realContent) return [];
  if (lang === 'ts') {
    plugins.push('typescript');
  }
  if (lang === 'js') {
    realContent = realContent.replace(/\n[^\S]*(@)/g, '// $1');
  }
  realContent = realContent.replace(/\n[^\S]*(import)/g, '// $1');
  // console.log(realContent);
  return babelParse.parse(realContent, {
    allowImportExportEverywhere: true,
    sourceType,
    sourceFilename: filePath,
    startLine: start,
    plugins
  });
}

function _parseVueTemplate (content) {
  let templateObj;
  try {
    templateObj = compiler.parseComponent(content);
  } catch {
    throw new Error('vue文件解析失败');
  }
  const { script, template } = templateObj;
  // console.log(html.parse(template.content));
  return {
    content: script && script.content,
    start: script && (_getLine(content, script.start) + 1) || 1,
    lang: script && script.lang
  }
}

function getErrorContext(err, src) {
  if (!err.loc || !err.loc.line || err.loc.column >= 100) return '';

  const line = src.split('\n')[err.loc.line - 1];
  const caret = ' '.repeat(err.loc.column) + '^';

  return `\n${line}\n${caret}`;
}

function _getLine (content, start) {
  return content.slice(0, start).split('\n').length;
}
