const fs = require('fs');
const JsInspect = require('jsinspect').Inspector;
const parser = require('./parser').parser;
const NodeUtils = require('jsinspect/lib/nodeUtils');

class VueInspector extends JsInspect {
  /* 
    this._filePaths    = filePaths || [];
    this._threshold    = opts.threshold || 30;
    this._identifiers  = opts.identifiers;
    this._literals     = opts.literals;
    this._minInstances = opts.minInstances || 2;
    this._map          = Object.create(null);
    this._fileContents = {};
    this._traversals   = {};
    this.numFiles      = this._filePaths.length;
  */
  constructor(filePaths, opts) {
    super(filePaths, opts);
  }

  run () {
    this.emit('start');
    this._filePaths.forEach(filePath => {
      const fileContent = fs.readFileSync(filePath, 'utf-8');
      this._fileContents[filePath] = fileContent.split('\n');
      let syntaxTree;
      try {
        syntaxTree = parser(fileContent, filePath);
      } catch (e) {
        return console.error(e.message);
      }
      this._traversals[filePath] = NodeUtils.getDFSTraversal(syntaxTree);
      this._walk(syntaxTree, (nodes) => this._insert(nodes));
    })
    this._analyze();
    this.emit('end');
  }
}

module.exports = VueInspector;